//-----------------------------------------------------------------------------
// VSMSGWDemoDlg.cpp
//
// Main file for PC RTCU Demo application 
//
//
//
// All sections in the code below with a "//==========" around it, is code that
// in some way has to do with the actual RTCU Communication Hub/VSMSGW DLL interaction.
// Other than that, it is just a plain Dialog Based skeleton made by 
// Visual Studio 6. (Added the status bar manually)
//
//-----------------------------------------------------------------------------

#include "stdafx.h"
#include "VSMSGWDemo.h"
#include "VSMSGWDemoDlg.h"
#include "SetupDlg.h"
#include "VSMSGW.h"
#include <process.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static UINT BASED_CODE indicators[] =
{
  ID_INDICATOR_STATUS,
  ID_INDICATOR_ACTIVITY,
  ID_INDICATOR_TIME
};


/////////////////////////////////////////////////////////////////////////////
// CVSMSGWDemoDlg dialog

CVSMSGWDemoDlg::CVSMSGWDemoDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CVSMSGWDemoDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CVSMSGWDemoDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
  hisnodeid=0;
}

void CVSMSGWDemoDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CVSMSGWDemoDlg)
	DDX_Control(pDX, IDC_COUNT, m_ctrcount);
	DDX_Control(pDX, IDC_ANALOG4, m_ctranalog4);
	DDX_Control(pDX, IDC_ANALOG3, m_ctranalog3);
	DDX_Control(pDX, IDC_ANALOG2, m_ctranalog2);
	DDX_Control(pDX, IDC_ANALOG1, m_ctranalog1);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CVSMSGWDemoDlg, CDialog)
	//{{AFX_MSG_MAP(CVSMSGWDemoDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_SETUP, OnSetup)
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_ON1, OnOn1)
	ON_BN_CLICKED(IDC_OFF1, OnOff1)
	ON_BN_CLICKED(IDC_ON2, OnOn2)
	ON_BN_CLICKED(IDC_OFF2, OnOff2)
	ON_BN_CLICKED(IDC_ON3, OnOn3)
	ON_BN_CLICKED(IDC_OFF3, OnOff3)
	ON_BN_CLICKED(IDC_ON4, OnOn4)
	ON_BN_CLICKED(IDC_OFF4, OnOff4)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CVSMSGWDemoDlg message handlers



//=================================================================================================
//-----------------------------------------------------------------------------
// This function gets called every time a Text SMS message is received from an
// RTCU unit. In this application, we use the data to update the bargraphs on 
// the screen 
//-----------------------------------------------------------------------------
static int _cdecl cbText(int HisNodeID, char *str, void* arg) {
  int ai[4];
  static int count=1;
  // Decode the text SMS message from the RTCU unit:
  // Format from RTCU is: 100,200,300,400
  sscanf(str, "%i,%i,%i,%i", &ai[0], &ai[1], &ai[2], &ai[3]);
  
  // Update the bargraph on screen
  ((CVSMSGWDemoDlg*)arg)->m_ctranalog1.SetPos(ai[0]);
  ((CVSMSGWDemoDlg*)arg)->m_ctranalog2.SetPos(ai[1]);
  ((CVSMSGWDemoDlg*)arg)->m_ctranalog3.SetPos(ai[2]);
  ((CVSMSGWDemoDlg*)arg)->m_ctranalog4.SetPos(ai[3]);
  
  // Update counter on screen
  CString s;
  s.Format("%i", count++);
  ((CVSMSGWDemoDlg*)arg)->m_ctrcount.SetWindowText(s);
  
  // Save the nodeid the data came from, this is used for sending ON/OFF messages from the program to the "correct" receiver
  // (This means that you can NOT send ON/OFF commands BEFORE that the program has received at least one update message from the RTCU,
  // otherwise the program don't know the nodeid of the RTCU unit !
  ((CVSMSGWDemoDlg*)arg)->hisnodeid=HisNodeID;
  return 0;
}
//=================================================================================================


//=================================================================================================
//-----------------------------------------------------------------------------
// This function gets called every time a PDU SMS message is received from an
// RTCU unit. In this application, we don't use PDU SMS messages, so just return 0
// to the sender, should it decide to send a PDU to us !
//-----------------------------------------------------------------------------
static int _cdecl cbPDU(int HisNodeID, unsigned char *data, int length, void* arg) {
  return 0;
}
//=================================================================================================

static char hatoi(const CString& ha)
{
  // Var
  char res = 0, i;

  // Iterate nibles
  for (i = 0; i < 2; i++) {
    // Move last nible
    res <<= 4;
    // Set nible
    if (ha.GetAt(i) >= '0' && ha.GetAt(i) <= '9')      res += (ha.GetAt(i) - '0');
    else if (ha.GetAt(i) >= 'a' && ha.GetAt(i) <= 'f') res += (ha.GetAt(i) - 'a' + 10);
    else if (ha.GetAt(i) >= 'A' && ha.GetAt(i) <= 'F') res += (ha.GetAt(i) - 'A' + 10);
  }

  // Completed
  return res;
}


BOOL CVSMSGWDemoDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
  

  m_bar.Create(this); //We create the status bar
  m_bar.SetIndicators(indicators,3); //Set the number of panes 

  CRect rect;
  GetClientRect(&rect);

  //Size the three panes
  m_bar.SetPaneInfo(0,ID_INDICATOR_STATUS, SBPS_NORMAL,rect.Width()-130);      
  m_bar.SetPaneInfo(1,ID_INDICATOR_ACTIVITY, SBPS_NORMAL, 12);      
  m_bar.SetPaneInfo(2,ID_INDICATOR_TIME,SBPS_STRETCH ,0);

  //This is where we actually draw it on the screen
  RepositionBars(AFX_IDW_CONTROLBAR_FIRST,AFX_IDW_CONTROLBAR_LAST,ID_INDICATOR_TIME);

  m_bar.SetPaneText(0,"");
  CTime t1;
  t1=CTime::GetCurrentTime();
  m_bar.SetPaneText(2,t1.Format("%H:%M:%S"));

  SetTimer(100,500,NULL);

  m_ctranalog1.SetRange(0,1023);
  m_ctranalog2.SetRange(0,1023);
  m_ctranalog3.SetRange(0,1023);
  m_ctranalog4.SetRange(0,1023);

  char key[9];
  strcpy(key, (LPCTSTR)AfxGetApp()->GetProfileString("Options", "GW Key", "AABBCCDD"));

  unsigned char cryptkey[16];
  memset(cryptkey, 0, sizeof(cryptkey));
  if (AfxGetApp()->GetProfileInt("Options", "GW ENCRYPTION", 0)) {
    // Var
    CString str, strList;
    int     i, pos;

    // Iterate cryptkey bytes
    strList = AfxGetApp()->GetProfileString("Options", "GW CRYPTKEY", "00000000-00000000-00000000-00000000");
    pos = 0;
    for (i = 0; i < 16; i++) {
      // Insert byte
      cryptkey[i] = hatoi(strList.Mid(pos, 2));

      // Calculate next position
      pos += 2;
      if ((i + 1) % 4 == 0) pos++;
    }
  }

  //=================================================================================================
  //-----------------------------------------------------------------------------
  // Initialize the VSMSGW library, establishing a connection to the RTCU Communication Hub
  // with the appropriate parameters
  //-----------------------------------------------------------------------------
  int rc=vsmsgwInitAdv(
                       AfxGetApp()->GetProfileInt("Options", "My nodeid", 4711),
                       (LPCTSTR)AfxGetApp()->GetProfileString("Options", "GW IP", "rtcu.dk"),
                       AfxGetApp()->GetProfileInt("Options", "GW Port", 5001),
                       key,
                       cryptkey
                      );
  vsmsgwSetSMSTextCB(cbText, this);
  vsmsgwSetSMSPDUCB(cbPDU, this);
  //=================================================================================================

	return TRUE;  // return TRUE  unless you set the focus to a control
}


//-----------------------------------------------------------------------------
// Update connection-status in the status panel
//-----------------------------------------------------------------------------
void CVSMSGWDemoDlg::OnTimer(UINT_PTR nIDEvent) 
{
  static bool old_fConnected=true; // The first time we arrive in this function, we are not connected anyway, so this forces an update of the display...

  if(nIDEvent==100) {

    CTime t1;
    t1=CTime::GetCurrentTime();
    m_bar.SetPaneText(2,t1.Format("%H:%M:%S %Y%m%d"));

    //=================================================================================================
    //-----------------------------------------------------------------------------
    // Query the VSMSGW DLL about the connection status to the RTCU Communication Hub
    //-----------------------------------------------------------------------------
    if (vsmsgwIsConnected() != old_fConnected) {
      old_fConnected=vsmsgwIsConnected();
      CString s;
      s.Format("%s", vsmsgwIsConnected() ? "Connected to RTCU Communication Hub" : "Connecting to RTCU Communication Hub....");
      m_bar.SetPaneText(0,s);
    }
    UpdateData(false);
  }
  //=================================================================================================

	CDialog::OnTimer(nIDEvent);
}


void CVSMSGWDemoDlg::OnOn1() 
{
  // Check if we have received at least one message from the RTCU unit, so we know which one to send the message to...
  if (hisnodeid==0)
    return;
  int rc;
  //-----------------------------------------------------------------------------
  // Send a "ON" Text SMS to the RTCU unit
  //-----------------------------------------------------------------------------
  //=================================================================================================
  vsmsgwSendSMS(hisnodeid,"ON 1", &rc);
  //=================================================================================================
}


void CVSMSGWDemoDlg::OnOff1() 
{
  // Check if we have received at least one message from the RTCU unit, so we know which one to send the message to...
  if (hisnodeid==0)
    return;
  int rc;
  //=================================================================================================
  //-----------------------------------------------------------------------------
  // Send a "OFF" Text SMS to the RTCU unit
  //-----------------------------------------------------------------------------
  vsmsgwSendSMS(hisnodeid,"OFF 1", &rc);
  //=================================================================================================
}


void CVSMSGWDemoDlg::OnOn2() 
{
  if (hisnodeid==0)
    return;
  int rc;
  //=================================================================================================
  vsmsgwSendSMS(hisnodeid,"ON 2", &rc);
  //=================================================================================================
}

void CVSMSGWDemoDlg::OnOff2() 
{
  if (hisnodeid==0)
    return;
  int rc;
  //=================================================================================================
  vsmsgwSendSMS(hisnodeid,"OFF 2", &rc);
  //=================================================================================================
}

void CVSMSGWDemoDlg::OnOn3() 
{
  if (hisnodeid==0)
    return;
  int rc;
  //=================================================================================================
  vsmsgwSendSMS(hisnodeid,"ON 3", &rc);
  //=================================================================================================
}

void CVSMSGWDemoDlg::OnOff3() 
{
  if (hisnodeid==0)
    return;
  int rc;
  //=================================================================================================
  vsmsgwSendSMS(hisnodeid,"OFF 3", &rc);
  //=================================================================================================
}

void CVSMSGWDemoDlg::OnOn4() 
{
  if (hisnodeid==0)
    return;
  int rc;
  //=================================================================================================
  vsmsgwSendSMS(hisnodeid,"ON 4", &rc);
  //=================================================================================================
}

void CVSMSGWDemoDlg::OnOff4() 
{
  if (hisnodeid==0)
    return;
  int rc;
  //=================================================================================================
  vsmsgwSendSMS(hisnodeid,"OFF 4", &rc);
  //=================================================================================================
}


// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.
void CVSMSGWDemoDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CVSMSGWDemoDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}


void CVSMSGWDemoDlg::OnCancel() 
{
  if (AfxMessageBox("Are you sure you want to exit the program ?", MB_ICONWARNING|MB_YESNO|MB_DEFBUTTON2)==IDYES) 
    CDialog::OnCancel();
}


void CVSMSGWDemoDlg::OnOK() 
{
	CDialog::OnOK();
}


//-----------------------------------------------------------------------------
// Modify connection parameters, and save them in registry...
//-----------------------------------------------------------------------------
void CVSMSGWDemoDlg::OnSetup() 
{
  // Var
  CSetupDlg dlg;
  CString   str, strList;
  int       i, pos;

  // Load configuration
  dlg.m_gwport=      AfxGetApp()->GetProfileInt(   "Options", "GW Port",   5001);
  dlg.m_gwip=        AfxGetApp()->GetProfileString("Options", "GW IP",     "gw.rtcu.dk");
  dlg.m_gwkey=       AfxGetApp()->GetProfileString("Options", "GW Key",    "AABBCCDD");
  dlg.m_mynodeid=    AfxGetApp()->GetProfileInt(   "Options", "My nodeid", 4711);
  dlg.m_enable_crypt=AfxGetApp()->GetProfileInt(   "Options", "GW ENCRYPTION", 0);
  strList=           AfxGetApp()->GetProfileString("Options", "GW CRYPTKEY", "00000000-00000000-00000000-00000000");

  // Iterate cryptkey bytes
  pos = 0;
  for (i = 0; i < 16; i++) {
    // Insert byte
    dlg.m_crypt_key[i] = hatoi(strList.Mid(pos, 2));

    // Calculate next position
    pos += 2;
    if ((i + 1) % 4 == 0) pos++;
  }

  // Change settings
  if (dlg.DoModal()==IDOK) {
    // Iterate cryptkey bytes
    strList.Empty();
    for (i = 0; i < 16; i++) {
      // Insert byte
      str.Format("%02X", dlg.m_crypt_key[i]);
      strList += str;
      // Divider?
      if ((i + 1) % 4 == 0 && (i + 1) != 16) {
        // Insert divider
        strList += _T("-");
      }
    }
    AfxGetApp()->WriteProfileInt(   "Options", "GW Port", dlg.m_gwport);
    AfxGetApp()->WriteProfileString("Options", "GW IP", dlg.m_gwip);
    AfxGetApp()->WriteProfileString("Options", "GW Key", dlg.m_gwkey);
    AfxGetApp()->WriteProfileInt(   "Options", "My nodeid", dlg.m_mynodeid);
    AfxGetApp()->WriteProfileInt(   "Options", "GW ENCRYPTION", dlg.m_enable_crypt);
    AfxGetApp()->WriteProfileString("Options", "GW CRYPTKEY", strList);
    AfxMessageBox("The program must be restarted in order to use any new settings !");
  }
}
